#include <cmath>
using std::fmod;

#include "timers.hh"

void timer::Run(double seconds, double length)
{
	if(active)
	{
		unsigned n = Advance(length, seconds);
		if(n)
		{
			unsigned newvalue = value + n;
			if(newvalue <= 255)
				value = newvalue;
			else
			{
				if(irqs)hit = true;
				value = reset + (newvalue-256) % (256-reset);
			}
		}
	}
}

void timer::Activate()
{
	active = true;
	value = reset;
}

void timer::Deactivate()
{
	active = false;
	timefraction = 0.0;
}

void timer::Set(unsigned char nv)
{
	value = reset = nv;
}

void Timers::Run(double seconds)
{
	timer1.Run(seconds,  80E-6);
	timer2.Run(seconds, 320E-6);
}

const unsigned char Timers::ReadStatus() const
{
	unsigned char t = 0;
	if(timer1.hit) t |= 0x40;
	if(timer2.hit) t |= 0x20;
	if(t)t |= 0x80;
	return t;
}

void Timers::WriteTimer1(unsigned char value)
{
	timer1.Set(value);
}

void Timers::WriteTimer2(unsigned char value)
{
	timer2.Set(value);
}

void Timers::WriteReg4(unsigned char value)
{
	if(value & 0x80)
	{
		timer1.hit = false;
		timer2.hit = false;
	}
	else
	{
		if(value & 1)timer1.Activate(); else timer1.Deactivate();
		if(value & 2)timer2.Activate(); else timer2.Deactivate();
		
		timer1.irqs = !(value & 0x40);
		timer2.irqs = !(value & 0x20);
	}
}
